import org.junit.Test;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;

public class TestMyArrayList {

    @Test
    public void testAddString() {
        MyArrayList<String> list = new MyArrayList<>();
        list.add("Privet");
        list.add("Poka");
        String expected = "Poka";
        String actual = list.get(1);
        assertEquals(expected, actual);
    }

    @Test
    public void testAddInteger() {
        MyArrayList<Integer> list = new MyArrayList<>();
        list.add(1243);
        list.add(456);
        Integer expected = 1243;
        Integer actual = list.get(0);
        assertEquals(expected, actual);
    }

    @Test
    public void testRemove() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 18; i++) {
            list.add(i * 3);
        }
        list.remove(7);
        Integer expected = 48;
        Integer actual = list.get(15);
        assertEquals(expected, actual);
    }

    @Test
    public void testRemove2() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 10; i++) {
            list.add(i * 3);
        }
        list.remove(1);
        list.remove(5);
        list.remove(6);
        Integer expected1 = 6;
        Integer actual1 = list.get(1);
        Integer expected2 = 15;
        Integer actual2 = list.get(4);
        Integer expected3 = 27;
        Integer actual3 = list.get(6);
        assertEquals(expected1, actual1);
        assertEquals(expected2, actual2);
        assertEquals(expected3, actual3);
    }

    @Test
    public void testSet() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 38; i++) {
            list.add(i * 3);
        }
        Integer actual1 = list.set(17, 278);
        Integer expected1 = 51;
        Integer expected = 278;
        Integer actual = list.get(17);
        assertEquals(expected1, actual1);
        assertEquals(expected, actual);
    }

    @Test
    public void testSetOutOfRange() throws IndexOutOfBoundsException {
        boolean thrown = false;
        try {
            MyArrayList<Integer> list = new MyArrayList<>();
            for (int i = 0; i < 15; i++) {
                list.add(i * 3);
            }
            list.set(17, 278);
        } catch (IndexOutOfBoundsException e) {
            thrown = true;
        }
        assertTrue(thrown);
    }

    @Test
    public void testSetBorderRange() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 15; i++) {
            list.add(i * 3);
        }
        int expected2 = list.set(14, 234);
        int actual2 = 42;
        int expected = 15;
        int actual = list.size();
        int expected1 = 234;
        int actual1 = list.get(14);
        assertEquals(expected, actual);
        assertEquals(expected1, actual1);
        assertEquals(expected2, actual2);
    }

    @Test
    public void testClear() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 15; i++) {
            list.add(i * 3);
        }
        list.clear();
        int expected = 0;
        int actual = list.size();
        assertEquals(expected, actual);
    }

    @Test
    public void testClearException() {
        boolean thrown = false;
        try {
            MyArrayList<Integer> list = new MyArrayList<>();
            for (int i = 0; i < 15; i++) {
                list.add(i * 2);
            }
            list.clear();
            list.get(0);
        } catch (IndexOutOfBoundsException e) {
            thrown = true;
        }
        assertTrue(thrown);
    }

    @Test
    public void testIsEmptyTrue() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 15; i++) {
            list.add(i * 3);
        }
        list.clear();
        assertTrue(list.isEmpty());
    }

    @Test
    public void testIsEmptyFalse() {
        MyArrayList<String> list = new MyArrayList<>();
        list.add("Lukas");
        assertFalse(list.isEmpty());
    }

    @Test
    public void testContainsFalse() {
        MyArrayList<String> list = new MyArrayList<>();
        list.add("Lukas");
        assertFalse(list.contains("Boo"));
    }

    @Test
    public void testContainsTrue() {
        MyArrayList<String> list = new MyArrayList<>();
        list.add("Fall Out Boy");
        assertTrue(list.contains("Fall Out Boy"));
    }

    @Test
    public void testContainsMiddleTrue() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 15; i++) {
            list.add(i * 3);
        }
        assertTrue(list.contains(33));
    }

    @Test
    public void testContainsNull() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 15; i++) {
            list.add(i * 3);
        }
        list.add(null);
        assertTrue(list.contains(null));
    }

    @Test
    public void testSize() {
        MyArrayList<Integer> list = new MyArrayList<>();
        for (int i = 0; i < 15; i++) {
            list.add(i * 3);
        }
        list.remove(12);
        list.remove(0);
        list.remove(9);
        int expected = 12;
        int actual = list.size();
        assertEquals(expected, actual);
    }
}
